
import 'dart:convert';
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:pia_framework_base/Popup/PIA_Alert.dart';
import 'package:pia_framework_base/Func/PIA_Func.dart';
import 'package:pia_framework_base/Variable/PIA_Global.dart';
import 'package:pia_framework_base/Popup/PIA_Loading.dart';
import 'package:pia_framework_base/Lang/PIA_Lang.dart';

// #docregion print-state
class PIA_API_Popup {
  static CancelToken token = new CancelToken();
  static Dio dio = new Dio();


  static Post_Return(BuildContext context,String url,Map<String,String> post, function_return) async {
    print("Post Url $url");
    print("Post value " + post.toString());

    var network = await PIA_Func.Check_network();
    if(!network) {
      PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      return;
    }

    token = new CancelToken();
    Dio dio = new Dio();
    dio.options.connectTimeout = connectTimeout;
    dio.options.receiveTimeout = receiveTimeout;

    Map<String, dynamic> fordata = Map<String, dynamic>();
    if (post != null) {
      post.forEach((String key, String value) {
        fordata[key] = value;
      });
    }


    PIA_Loading.show(context);
    Response response = await dio.post(
      url,
      data: new FormData.fromMap(fordata),
      cancelToken: token,
      onSendProgress: (int sent, int total) {
        print(" $sent $total");
      },
    ).catchError((err) {
      token = null;

      PIA_Loading.hide(context);
      DioError error = err;
      print("Catch Error " + error.message);
      print("Error type ");
      print(error.type);

      if (error.type == DioErrorType.response) {
        PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url);
      }

      if (error.type == DioErrorType.connectTimeout) {
        PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      }

      if (error.type == DioErrorType.other) {
        if(error.message.contains("FormatException"))
        { PIA_Alert.Error(context,api_program_error[PIA_Lang.lang_id],url + "\n\n" + error.message); }
        else
        if(error.message.contains("HttpException"))
        { PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);  }
        else
        { PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url); }
      }

      if (error.type == DioErrorType.receiveTimeout) {
        PIA_Alert.Error(context,"",api_connect_error_slow[PIA_Lang.lang_id]);
      }
      return null;

    });
    token = null;

    PIA_Loading.hide(context);
    if (response != null && response.statusCode == 200) {
      print(response.toString());
      print("status :" + response.statusCode.toString());

      var js = jsonDecode(response.toString());
      if (js["status"] == 0 && js["err_message"]
          .toString()
          .isNotEmpty) {
        PIA_Alert.Error(context, api_error[PIA_Lang.lang_id] , js["err_message"].toString());
      }
      else {
        function_return(js["sdata"], js["data"]);
        }
      }

  }


  static Post(BuildContext context,String url,Map<String,String> post,PIA_API_CallBack callback) async {
    print("Post Url $url");
    print("Post value " + post.toString());

    var network = await PIA_Func.Check_network();
    if(!network) {
      PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      return;
    }

    token = new CancelToken();
    Dio dio = new Dio();
    dio.options.connectTimeout = connectTimeout;
    dio.options.receiveTimeout = receiveTimeout;

    Map<String, dynamic> fordata = Map<String, dynamic>();
    if (post != null) {
      post.forEach((String key, String value) {
        fordata[key] = value;
      });
    }


    PIA_Loading.show(context);
    Response response = await dio.post(
      url,
      data: new FormData.fromMap(fordata),
      cancelToken: token,
      onSendProgress: (int sent, int total) {
        print(" $sent $total");
      },
    ).catchError((err) {
      token = null;

      PIA_Loading.hide(context);
      DioError error = err;
      print("Catch Error " + error.message);
      print("Error type ");
      print(error.type);

      if (error.type == DioErrorType.response) {
        PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url);
      }

      if (error.type == DioErrorType.connectTimeout) {
        PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      }

      if (error.type == DioErrorType.other) {
        if(error.message.contains("FormatException"))
        { PIA_Alert.Error(context,api_program_error[PIA_Lang.lang_id],url + "\n\n" + error.message); }
        else
        if(error.message.contains("HttpException"))
        { PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);  }
        else
        { PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url); }
      }

      if (error.type == DioErrorType.receiveTimeout) {
        PIA_Alert.Error(context,"",api_connect_error_slow[PIA_Lang.lang_id]);
      }


      if (callback != null) {
        callback.API_Error(error.type);
        }
      return null;

    });
    token = null;

    PIA_Loading.hide(context);
    if (response != null && response.statusCode == 200) {
      print(response.toString());
      print("status :" + response.statusCode.toString());

      var js = jsonDecode(response.toString());
      if (js["status"] == 0 && js["err_message"]
          .toString()
          .isNotEmpty) {
        PIA_Alert.Error(context, api_error[PIA_Lang.lang_id] , js["err_message"].toString());
      }
      else {
        if (callback != null) {
          callback.API_Success(js["sdata"], js["data"]);
        }
      }
    }
  }

  static Upload(BuildContext context,String url,Map<String,String> post, Map<String,File> upload,PIA_API_CallBack callback) async {
    var network = await PIA_Func.Check_network();
    if(!network) {
      PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      return;
    }

    final number = new ValueNotifier<String>("0%");
    _show_loading_box(context,number);

    print("Post Url $url");
    print("Post value " + post.toString());
    print("Post File " + upload.toString());

    token = new CancelToken();
    Dio dio = new Dio();
    dio.options.connectTimeout = connectTimeout;

    Map<String, dynamic> fordata = Map<String, dynamic>();
    if (post != null) {
      post.forEach((String key, String value) {
        fordata[key] = value;
      });
    }

    var uploadlist = upload.entries.toList();
    for(var v in uploadlist)
    {  String key = v.key;
       File file = v.value;
       fordata[key] =  await MultipartFile.fromFile(file.path, filename: file.path.split("/").last);
    }


    var formData = new FormData.fromMap(fordata);

    Response response = await dio.post(
      url,
      data: formData,
      cancelToken: token,
      onSendProgress: (int sent, int total) {
      int loading = (sent * 100/total).round();
      number.value = "$loading" + "%";
      },
    ).catchError((err) async {
      token = null;
      DioError error = err;
      print("Catch Error " + error.message);
      print("Error type ");
      print(error.type);

      if(error.type != DioErrorType.cancel)
        { Navigator.pop(context); }

      if (error.type == DioErrorType.response) {
        PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url);
      }

      if (error.type == DioErrorType.connectTimeout) {
        PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);
      }

      if (error.type == DioErrorType.other) {
        if(error.message.contains("FormatException"))
        { PIA_Alert.Error(context,api_program_error[PIA_Lang.lang_id],url + "\n\n" + error.message); }
        else
          if(error.message.contains("HttpException"))
              { PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],api_nonetwork[PIA_Lang.lang_id]);  }
          else
              { PIA_Alert.Error(context,api_link_error[PIA_Lang.lang_id],url); }
      }


      if (error.type == DioErrorType.receiveTimeout) {
        //PIA_Alert.Error(context,"",api_connect_error_slow[PIA_Lang.lang_id]);

      }


      if (callback != null && error.type != DioErrorType.cancel) {
        callback.API_Error(error.type);
      }
      return null;
    });
    token = null;
    Navigator.pop(context);
    if (response != null && response.statusCode == 200) {
      print(response.toString());
      print("status :" + response.statusCode.toString());

      var js = jsonDecode(response.toString());
      if (js["status"] == 0 && js["err_message"]
          .toString()
          .isNotEmpty) {
        print("API Return Status 0 ");
        PIA_Alert.Error(context,api_error[PIA_Lang.lang_id],js["err_message"]);
      }
      else {
        if (callback != null) {
          callback.API_Success(js["sdata"], js["data"]);
        }
      }
    }
  }


static void _show_loading_box(BuildContext context,ValueListenable<String> number) {
  showDialog<String>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        print("Show Dialog Builder");
        var dialog =  WillPopScope(
            onWillPop: () { print("Back Press");},
        child:new CupertinoAlertDialog (
          title: new Text(api_uploading[PIA_Lang.lang_id]),
          content: ValueListenableBuilder<String>(
          valueListenable: number,
          builder: (context, value, child) {
          print("Value Listen Builder");
          return Text(value);
        }),
          actions: <Widget>[
            CupertinoDialogAction(isDefaultAction: true,
              child: Text(api_cancel[PIA_Lang.lang_id]),
              onPressed: () {
                token.cancel("cancelled");
                print("Dio Clear");
              },
            )
          ],
        ));
        return dialog;
      }
   );
 }

} // end of class


